@@ -3,7 +3,8 @@
  *  XVID MPEG-4 VIDEO CODEC
  *  - Native API implementation  -
  *
- *  Copyright(C) 2001-2004 Peter Ross <pross@xvid.org>
+ *  Copyright(C) 2001-2011 Peter Ross <pross@xvid.org>
+ *               2002-2011 Michael Militzer <isibaar@xvid.org>
  *
  *  This program is free software ; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,7 +20,7 @@
  *  along with this program ; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * $Id: xvid.c,v 1.77.2.5 2009/05/25 08:09:23 Isibaar Exp $
+ * $Id: xvid.c 2011 2011-05-23 07:47:37Z Isibaar $
  *
  ****************************************************************************/
 
@@ -29,7 +30,23 @@
 #include <time.h>
 
 #if !defined(_WIN32)
-  #include <unistd.h>
+#include <unistd.h>
+#endif
+
+#if defined(__APPLE__) && defined(__MACH__) && !defined(_SC_NPROCESSORS_CONF)
+#include <sys/types.h>
+#include <sys/sysctl.h>
+#ifdef MAX
+#undef MAX
+#endif
+#ifdef MIN
+#undef MIN
+#endif
+#endif
+
+#if defined(__amigaos4__)
+#include <exec/exec.h>
+#include <proto/exec.h>
 #endif
 
 #include "xvid.h"
@@ -58,7 +75,7 @@ unsigned int xvid_debug = 0; /* xvid debug mask */
 
 #if (defined(ARCH_IS_IA32) || defined(ARCH_IS_X86_64)) && defined(_MSC_VER)
 #	include <windows.h>
-#elif defined(ARCH_IS_IA32) || defined(ARCH_IS_X86_64) || defined(ARCH_IS_PPC)
+#elif defined(ARCH_IS_IA32) || defined(ARCH_IS_X86_64) || (defined(ARCH_IS_PPC) && !defined(__amigaos4__))
 #	include <signal.h>
 #	include <setjmp.h>
 
@@ -71,7 +88,6 @@ unsigned int xvid_debug = 0; /* xvid debug mask */
 	}
 #endif
 
-
 /*
  * Calls the funcptr, and returns whether SIGILL (illegal instruction) was
  * signalled
@@ -94,7 +110,7 @@ sigill_check(void (*func)())
 	}
 	return(0);
 }
-#elif defined(ARCH_IS_IA32) || defined(ARCH_IS_X86_64) || defined(ARCH_IS_PPC)
+#elif defined(ARCH_IS_IA32) || defined(ARCH_IS_X86_64) || (defined(ARCH_IS_PPC) && !defined(__amigaos4__))
 static int
 sigill_check(void (*func)())
 {
@@ -146,8 +162,18 @@ detect_cpu_flags(void)
 #endif
 
 #if defined(ARCH_IS_PPC)
+#if defined(__amigaos4__)
+        {
+                uint32_t vector_unit = VECTORTYPE_NONE;
+                IExec->GetCPUInfoTags(GCIT_VectorUnit, &vector_unit, TAG_END);
+                if (vector_unit == VECTORTYPE_ALTIVEC) {
+                        cpu_flags |= XVID_CPU_ALTIVEC;
+                }
+        }
+#else
 	if (!sigill_check(altivec_trigger))
 		cpu_flags |= XVID_CPU_ALTIVEC;
+#endif
 #endif
 
 	return cpu_flags;
@@ -155,7 +181,7 @@ detect_cpu_flags(void)
 
 
 /*****************************************************************************
- * XviD Init Entry point
+ * Xvid Init Entry point
  *
  * Well this function initialize all internal function pointers according
  * to the CPU features forced by the library client or autodetected (depending
@@ -180,7 +206,6 @@ int xvid_gbl_init(xvid_gbl_init_t * init)
 	cpu_flags = (init->cpu_flags & XVID_CPU_FORCE) ? init->cpu_flags : detect_cpu_flags();
 
 	/* Initialize the function pointers */
-	idct_int32_init();
 	init_vlc_tables();
 
 	/* Fixed Point Forward/Inverse DCT transformations */
@@ -303,15 +328,19 @@ int xvid_gbl_init(xvid_gbl_init_t * init)
 	yv12_to_uyvyi   = yv12_to_uyvyi_c;
 
 	/* Functions used in motion estimation algorithms */
-	calc_cbp   = calc_cbp_c;
-	sad16      = sad16_c;
-	sad8       = sad8_c;
-	sad16bi    = sad16bi_c;
-	sad8bi     = sad8bi_c;
-	dev16      = dev16_c;
-	sad16v	   = sad16v_c;
-	sse8_16bit = sse8_16bit_c;
-	sse8_8bit  = sse8_8bit_c;
+	calc_cbp      = calc_cbp_c;
+	sad16         = sad16_c;
+	sad8          = sad8_c;
+	sad16bi       = sad16bi_c;
+	sad8bi        = sad8bi_c;
+	dev16         = dev16_c;
+	sad16v        = sad16v_c;
+	sse8_16bit    = sse8_16bit_c;
+	sse8_8bit     = sse8_8bit_c;
+
+	sseh8_16bit   = sseh8_16bit_c;
+	coeff8_energy = coeff8_energy_c;
+	blocksum8     = blocksum8_c;
 
 	init_GMC(cpu_flags);
 
@@ -541,8 +570,13 @@ int xvid_gbl_init(xvid_gbl_init_t * init)
 		dequant_h263_inter = dequant_h263_inter_sse2;
 
 		/* SAD operators */
-		sad16    = sad16_sse2;
-		dev16    = dev16_sse2;
+		sad16       = sad16_sse2;
+		dev16       = dev16_sse2;
+
+		/* PSNR-HVS-M distortion metric */
+		sseh8_16bit   = sseh8_16bit_sse2;
+		coeff8_energy = coeff8_energy_sse2;
+		blocksum8     = blocksum8_sse2;
 
 		/* DCT operators */
 		fdct = fdct_sse2_skal;
@@ -665,9 +699,9 @@ xvid_gbl_info(xvid_gbl_info_t * info)
 		return XVID_ERR_VERSION;
 
 	info->actual_version = XVID_VERSION;
-	info->build = "xvid-1.2.2";
+	info->build = "xvid-1.3.2";
 	info->cpu_flags = detect_cpu_flags();
-	info->num_threads = 0;
+	info->num_threads = 0; /* single-thread */
 
 #if defined(_WIN32)
 
@@ -677,10 +711,33 @@ xvid_gbl_info(xvid_gbl_info_t * info)
 	info->num_threads = siSysInfo.dwNumberOfProcessors; /* number of _logical_ cores */
   }
 
-#else
+#elif defined(_SC_NPROCESSORS_CONF) /* should be available on Apple too actually */
 
   info->num_threads = sysconf(_SC_NPROCESSORS_CONF);	
 
+#elif defined(__APPLE__) && defined(__MACH__)
+
+  {
+    size_t len;
+    int    mib[2], ncpu;
+
+    mib[0] = CTL_HW;
+    mib[1] = HW_NCPU;
+    len    = sizeof(ncpu);
+    if (sysctl(mib, 2, &ncpu, &len, NULL, 0) == 0)
+      info -> num_threads = ncpu;
+    else
+      info -> num_threads = 1;
+  }
+
+#elif defined(__amigaos4__)
+
+  {
+    uint32_t num_threads = 1;
+    IExec->GetCPUInfoTags(GCIT_NumberOfCPUs, &num_threads, TAG_END);
+    info->num_threads = num_threads;
+  }
+
 #endif
 
 	return 0;
@@ -717,6 +774,15 @@ xvid_gbl_convert(xvid_gbl_convert_t* convert)
 						(uint8_t**)convert->output.plane, convert->output.stride,
 						convert->output.csp, convert->interlacing);
 			break;
+		
+		case XVID_CSP_INTERNAL :
+			img.y = (uint8_t*)convert->input.plane[0];
+			img.u = (uint8_t*)convert->input.plane[1];
+			img.v = (uint8_t*)convert->input.plane[2];
+			image_output(&img, width, height, convert->input.stride[0],
+						(uint8_t**)convert->output.plane, convert->output.stride,
+						convert->output.csp, convert->interlacing);
+			break;
 
 		default :
 			return XVID_ERR_FORMAT;
@@ -728,7 +794,7 @@ xvid_gbl_convert(xvid_gbl_convert_t* convert)
 }
 
 /*****************************************************************************
- * XviD Global Entry point
+ * Xvid Global Entry point
  *
  * Well this function initialize all internal function pointers according
  * to the CPU features forced by the library client or autodetected (depending
@@ -761,7 +827,7 @@ xvid_global(void *handle,
 }
 
 /*****************************************************************************
- * XviD Native decoder entry point
+ * Xvid Native decoder entry point
  *
  * This function is just a wrapper to all the option cases.
  *
@@ -793,7 +859,7 @@ xvid_decore(void *handle,
 
 
 /*****************************************************************************
- * XviD Native encoder entry point
+ * Xvid Native encoder entry point
  *
  * This function is just a wrapper to all the option cases.
  *
